MODEL1_DATA_SIZE = 30
MODEL2_DATA_SIZE = 30
MODELOMCHILD_DATA_SIZE = 30

USERNAME_ADMIN = "testadmin"
PASSWORD_ADMIN = "password"
MAX_PAGE_SIZE = 25
USERNAME_READONLY = "readonly"
PASSWORD_READONLY = "readonly"
INVALID_LOGIN_STRING = "Invalid login"
