{% extends "appbuilder/base.html" %}

{% block content %}
<center>
<h2>{{_('Welcome')}} {{first_name}} {{last_name}}</h2>

<p>{{_('Your user is activated you can now proceed to login')}}</p>
<center>

{% endblock %}
