{% extends 'appbuilder/baselayout.html' %}

{% block head_css %}
    {{ super() }}
    <link href="{{url_for('static',filename='css/clean-blog.min.css')}}" rel="stylesheet">
{% endblock %}
{% block head_js %}
    {{ super() }}
    <script src="{{url_for('static',filename='js/clean-blog.min.js')}}"></script>
{% endblock %}
